@@ -35,7 +35,7 @@ import unipath
 from unipath import pathof
 import unicodedata
 
-_launcher_version=20160130
+_launcher_version=20160605
 
 _PKG_VER = re.compile(r'''<\s*package[^>]*version\s*=\s*["']([^'"]*)['"][^>]*>''',re.IGNORECASE)
 
@@ -57,10 +57,13 @@ ext_mime_map = {
                 '.xhtml': 'application/xhtml+xml',
                 '.html' : 'application/xhtml+xml',
                 '.otf'  : 'application/vnd.ms-opentype',
+                #'.otf'  : 'application/x-font-opentype',
                 '.ttf'  : 'application/x-font-ttf',
                 '.woff' : 'application/font-woff',
                 '.mp3'  : 'audio/mpeg',
+                '.m4a'  : 'audio/mp4',
                 '.mp4'  : 'video/mp4',
+                '.m4v'  : 'video/mp4',
                 '.css'  : 'text/css',
                 '.ncx'  : 'application/x-dtbncx+xml',
                 '.xml'  : 'application/oebs-page-map+xml',
@@ -68,9 +71,8 @@ ext_mime_map = {
                 '.smil' : 'application/smil+xml',
                 '.pls'  : 'application/pls-xml',
                 '.js'   : 'text/javascript',
-                '.epub' : 'application/epub+zip',
                 #'.js'   : 'application/javascript',
-                #'.otf'  : 'application/x-font-opentype',
+                '.epub' : 'application/epub+zip',
                 }
 
 mime_base_map = {
@@ -92,6 +94,7 @@ mime_base_map = {
                 'application/oebps-package+xml' : '',
                 'application/oebs-page-map+xml' : 'Misc',
                 'application/smil+xml'          : 'Misc',
+                'text/javascript'               : 'Misc',
                 'application/pls-xml'           : 'Misc',
                 }
 
@@ -141,6 +144,7 @@ class Wrapper(object):
         self.spine_ppd = None
         self.spine = []
         self.guide = []
+        self.bindings = []
         self.package_tag = None
         self.epub_version = None
         # self.metadata_attr = None
@@ -161,6 +165,7 @@ class Wrapper(object):
             self.guide = op.get_guide()
             self.package_tag = op.get_package_tag()
             self.epub_version = op.get_epub_version()
+            self.bindings = op.get_bindings()
             # self.metadata = op.get_metadata()
             # self.metadata_attr = op.get_metadata_attr()
             self.metadataxml = op.get_metadataxml()
@@ -211,7 +216,7 @@ class Wrapper(object):
 
     def build_manifest_xml(self):
         manout = []
-        manout.append('<manifest>\n')
+        manout.append('  <manifest>\n')
         for id in sorted(self.id_to_mime):
             href = quoteurl(self.id_to_href[id])
             mime = self.id_to_mime[id]
@@ -227,8 +232,8 @@ class Wrapper(object):
             overlay = self.id_to_over[id]
             if overlay is not None:
                 over = ' media-overlay="%s"' % overlay
-            manout.append('<item id="%s" href="%s" media-type="%s"%s%s%s />\n' % (id, href, mime, props, fall, over))
-        manout.append('</manifest>\n')
+            manout.append('    <item id="%s" href="%s" media-type="%s"%s%s%s />\n' % (id, href, mime, props, fall, over))
+        manout.append('  </manifest>\n')
         return "".join(manout)
 
     def build_spine_xml(self):
@@ -244,7 +249,7 @@ class Wrapper(object):
         pagemapid = self.getpagemapid()
         if pagemapid is not None:
             map = ' page-map="%s"' % pagemapid
-        spineout.append('<spine%s%s%s>\n' %(ppd, ncx, map))
+        spineout.append('  <spine%s%s%s>\n' %(ppd, ncx, map))
         for (id, linear, properties) in self.spine:
             lin = ''
             if linear is not None:
@@ -252,19 +257,29 @@ class Wrapper(object):
             props = ''
             if properties is not None:
                 props = ' properties="%s"' % properties
-            spineout.append('<itemref idref="%s"%s%s/>\n' % (id, lin, props))
-        spineout.append('</spine>\n')
+            spineout.append('    <itemref idref="%s"%s%s/>\n' % (id, lin, props))
+        spineout.append('  </spine>\n')
         return "".join(spineout)
 
     def build_guide_xml(self):
         guideout = []
-        guideout.append('<guide>\n')
-        for (type, title, href) in self.guide:
-            href = quoteurl(href)
-            guideout.append('<reference type="%s" href="%s" title="%s"/>\n' % (type, href, title))
-        guideout.append('</guide>\n')
+        if len(self.guide) > 0 or self.epub_version.startswith('2'):
+            guideout.append('  <guide>\n')
+            for (type, title, href) in self.guide:
+                href = quoteurl(href)
+                guideout.append('    <reference type="%s" href="%s" title="%s"/>\n' % (type, href, title))
+            guideout.append('  </guide>\n')
         return "".join(guideout)
 
+    def build_bindings_xml(self):
+        bindout = []
+        if len(self.bindings) > 0 and self.epub_version.startswith('3'):
+            bindout.append('  <bindings>\n')
+            for (mtype, handler) in self.bindings:
+                bindout.append('    <mediaType media-type="%s" handler="%s"/>\n' % (mtype, handler))
+            bindout.append('  </bindings>\n')
+        return "".join(bindout)
+
     def build_opf(self):
         data = []
         data.append('<?xml version="1.0" encoding="utf-8" standalone="yes"?>\n')
@@ -273,6 +288,7 @@ class Wrapper(object):
         data.append(self.build_manifest_xml())
         data.append(self.build_spine_xml())
         data.append(self.build_guide_xml())
+        data.append(self.build_bindings_xml())
         data.append('</package>\n')
         return "".join(data)
 
@@ -351,6 +367,24 @@ class Wrapper(object):
         self.spine = spine
         self.modified['OEBPS/content.opf'] = 'file'
 
+    def getbindings_epub3(self):
+        return self.bindings
+
+    def setbindings_epub3(self, new_bindings):
+        bindings = []
+        for (mtype, handler) in new_bindings:
+            mtype = unicode_str(mtype)
+            handler = unicode_str(handler)
+            if mtype is None or mtype == "":
+                continue
+            if handler is None or handler == "":
+                continue
+            if handler not in self.id_to_href:
+                raise WrapperException('Handler not in Manifest')
+            bindings.append((mtype, handler))
+        self.bindings = bindings
+        self.modified['OEBPS/content.opf'] = 'file'
+
     def spine_insert_before(self, pos, sid, linear, properties=None):
         sid = unicode_str(sid)
         linear = unicode_str(linear)
